// Copyright (c) 2021-2022, The Tor Project, Inc.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package whatsapp

import (
	"context"
	"fmt"
	_ "github.com/mattn/go-sqlite3"
	"github.com/mdp/qrterminal"
	"gitlab.torproject.org/tpo/anti-censorship/rdsys/internal"
	"gitlab.torproject.org/tpo/anti-censorship/rdsys/pkg/usecases/distributors/gettor"
	"go.mau.fi/whatsmeow"
	waProto "go.mau.fi/whatsmeow/binary/proto"
	"go.mau.fi/whatsmeow/store/sqlstore"
	"go.mau.fi/whatsmeow/types"
	"go.mau.fi/whatsmeow/types/events"
	waLog "go.mau.fi/whatsmeow/util/log"
	"google.golang.org/protobuf/proto"
	"log"
	"os"
	"os/signal"
	"strings"
	"syscall"
)

func InitFrontend(cfg *internal.Config) {
	// Connect to the WhatsApp account and set up event handlers
	err := Connect(cfg)
	if err != nil {
		log.Fatalf("error connecting to WhatsApp: %s", err)
	}

	// Wait for a signal to gracefully disconnect
	signalChan := make(chan os.Signal, 1)
	signal.Notify(signalChan, syscall.SIGINT, syscall.SIGTERM)
	<-signalChan

	// Disconnect from the WhatsApp account
	Disconnect()
}

var client *whatsmeow.Client

func Connect(cfg *internal.Config) error {
	// Initialize the database logger
	dbLog := waLog.Stdout("Database", "DEBUG", true)

	//Initialize the database container
	container, err := sqlstore.New("sqlite3", "file:"+cfg.Distributors.Whatsapp.SessionFile+"?_foreign_keys=on", dbLog)
	if err != nil {
		return err
	}
	// Get the first device from the container
	device, err := container.GetFirstDevice()
	if err != nil {
		return err
	}
	// Initialize the client
	clientLog := waLog.Stdout("Client", "INFO", true)
	c := whatsmeow.NewClient(device, clientLog)
	//Set the event handlers
	SetHandlers(c, cfg)
	// Connect to WhatsApp
	if c.Store.ID == nil {
		//If the device is not logged in, get the QR code and wait for the user to scan it
		qrChan, _ := c.GetQRChannel(context.Background())
		err = c.Connect()
		if err != nil {
			return err
		}

		for channelItem := range qrChan {
			if channelItem.Event == "code" {
				// Render the QR code here
				qrterminal.GenerateHalfBlock(channelItem.Code, qrterminal.L, os.Stdout)
				// or just manually `echo 2@... | qrencode -t ansiutf8` in a terminal
				fmt.Println("Scan the QR code above to log in", channelItem.Code)
				// you can use the channelItem to render QR code or send it via HTTP, etc
			} else {
				fmt.Println("Login Success", channelItem.Event)
				break
			}
		}
	} else {
		// Already logged in, just connect
		err = c.Connect()
		fmt.Println("Login Success")
		if err != nil {
			return err
		}
	}
	// Store the client for later use
	client = c
	return nil
}

func Disconnect() {
	client.Disconnect()
}

func SetHandlers(c *whatsmeow.Client, cfg *internal.Config) {
	// Get the GettorDistributor
	gettorDistributor := gettor.GettorDistributor{}
	gettorDistributor.Init(cfg)

	c.AddEventHandler(func(evt interface{}) {
		eventHandler(evt, c, &gettorDistributor)
	})
}

func eventHandler(evt interface{}, client *whatsmeow.Client, distributor *gettor.GettorDistributor) {
	supportedPlatforms := distributor.SupportedPlatforms()
	switch v := evt.(type) {
	case *events.Message:
		fmt.Println("Received a message!", v.Message.GetConversation())
		platform := strings.ToLower(v.Message.GetConversation())
		linkSent := false

		// Check if the platform is one of the supported platforms
		if contains(supportedPlatforms, platform) {
			links := distributor.GetLinks(platform)
			// Send the links to the recipient via WhatsApp
			for _, link := range links {
				if err := SendMessage(link.Link, client, v.Info.Chat); err != nil {
					log.Println(err)
				}
				log.Printf("Sent link %q for platform %q to recipient %q via WhatsApp\n", link.Link, platform, v.Info.Chat.User)
			}
			linkSent = true
		}

		// If no link was sent, send a message with the list of platforms
		if !linkSent {
			platformList := strings.Join(supportedPlatforms, ", ")
			message := fmt.Sprintf("Supported platforms: %s", platformList)
			if err := SendMessage(message, client, v.Info.Chat); err != nil {
				log.Println(err)
			}
		}
	}
}

func SendMessage(message string, client *whatsmeow.Client, receiver types.JID) error {
	_, err := client.SendMessage(receiver, "", &waProto.Message{
		Conversation: proto.String(message),
	})
	if err != nil {
		return err
	}
	return nil
}

func contains(platformSlice []string, elem string) bool {
	for _, platform := range platformSlice {
		if platform == elem {
			return true
		}
	}
	return false
}
